/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.renderer;

import jalview.api.AlignViewportI;
import jalview.datamodel.SequenceI;

import java.util.ArrayList;
import java.util.List;

/**
 * Calculate and display alignment rulers
 * 
 * @author jprocter
 *
 */
public class ScaleRenderer
{
  /**
   * calculate positions markers on the alignment ruler
   * 
   * @return List { Object { .. } } Boolean: true/false for major/minor mark,
   *         Integer: marker position in alignment column coords, String: null
   *         or a String to be rendered at the position.
   */
  public static List<Object[]> calculateMarks(AlignViewportI av,
          int startx, int endx)
  {
    new ArrayList<Object[]>();

    int scalestartx = (startx / 10) * 10;

    SequenceI refSeq = av.getAlignment().getSeqrep();
    int refSp = 0, refEp = -1, refStart = 0, refEnd = -1, refStartI = 0, refEndI = -1;
    if (refSeq != null)
    {
      // find bounds and set origin appopriately
      // locate first visible position for this sequence
      int[] refbounds = av.getColumnSelection()
              .locateVisibleBoundsOfSequence(refSeq);

      refSp = refbounds[0];
      refEp = refbounds[1];
      refStart = refbounds[2];
      refEnd = refbounds[3];
      refStartI = refbounds[4];
      refEndI = refbounds[5];
      scalestartx = refSp + ((scalestartx - refSp) / 10) * 10;
    }

    if (refSeq == null && scalestartx % 10 == 0)
    {
      scalestartx += 5;
    }
    List<Object[]> marks = new ArrayList<Object[]>();
    String string;
    int maxX = 0, refN, iadj;
    // todo: add a 'reference origin column' to set column number relative to
    for (int i = scalestartx; i < endx; i += 5)
    {
      Object[] amark = new Object[3];
      if (((i - refSp) % 10) == 0)
      {
        if (refSeq == null)
        {
          iadj = av.getColumnSelection().adjustForHiddenColumns(i - 1) + 1;
          string = String.valueOf(iadj);
        }
        else
        {
          iadj = av.getColumnSelection().adjustForHiddenColumns(i - 1);
          refN = refSeq.findPosition(iadj);
          // TODO show bounds if position is a gap
          // - ie L--R -> "1L|2R" for
          // marker
          if (iadj < refStartI)
          {
            string = String.valueOf(iadj - refStartI);
          }
          else if (iadj > refEndI)
          {
            string = "+" + String.valueOf(iadj - refEndI);
          }
          else
          {
            string = String.valueOf(refN) + refSeq.getCharAt(iadj);
          }
        }
        amark[0] = Boolean.TRUE;
        amark[1] = Integer.valueOf(i - startx - 1);
        amark[2] = string;

      }
      else
      {
        amark[0] = Boolean.FALSE;
        amark[1] = Integer.valueOf(i - startx - 1);
        amark[2] = null;
      }
      marks.add(amark);
    }
    return marks;
  }

}
